require 'deep_merge'

module Wizypay
  class ApiClient
    class << self
      attr_accessor :api_endpoint, :api_key, :api_secret

      def post(path, payload = {})
        execute(path, { method: :post, payload: payload })
      end

      def put(path, payload = {})
        execute(path, { method: :put, payload: payload })
      end

      def get(path, params = {})
        execute(path, { method: :get, headers: { params: params } })
      end

      def delete(path)
        execute(path, { method: :delete })
      end

      def execute(path, req_params)
        req_params.deep_merge!(
          url: "#{api_endpoint}#{path}",
          headers: { accept: :json }
        )
        req = RestClient::Request.new(req_params)
        req.sign!(api_key, api_secret)
        parse_response req.execute
      end

      private

      def parse_response(response)
        Hash[JSON::parse(response).map { |k,v| [k.to_sym, v] }]
      rescue JSON::ParserError
        response
      end
    end
  end
end
