module Wizypay
  module CardMethods
    include SimpleHmac::Helper

    module ClassMethods
      attr_accessor :base_path

      def where(q)
        raw = ApiClient.get(base_path, q)
        Collection.new(self, raw[:data], raw[:meta])
      end

      def all(q = {})
        where(q)
      end

      def find(id)
        new(ApiClient.get("#{base_path}/#{id}")[:data])
      end

      def create(reference, amount, currency, merchant_id, user)
        user = user.to_json if user.is_a?(Hash)
        new(amount: amount, currency: currency, merchant_id: merchant_id, reference: reference, user: user).tap do |card|
          card.save
        end
      end

      def cancel(reference)
        new(reference: reference).cancel
      end

      def refund(reference, amount)
        new(reference: reference).refund(amount)
      end

      def url(reference)
        new(reference: reference).url
      end
    end

    def base_path
      self.class.base_path
    end

    def cancel
      ApiClient.post("#{base_path}/#{URI::encode_www_form_component reference}/cancel")
    end

    def refund(amount)
      ApiClient.post("#{base_path}/#{URI::encode_www_form_component reference}/refund", amount: amount)
    end

    def url
      timestamp    = Time.now.utc.httpdate
      security     = sign_string("#{reference}\n#{timestamp}", ApiClient.api_secret)
      query_string = {
          key:       ApiClient.api_key,
          timestamp: timestamp,
          reference: reference,
          security:  security
      }.map { |k, v| "#{k}=#{URI::encode_www_form_component(v)}" }.join('&')
      "#{ApiClient.api_endpoint}/widget?#{query_string}"
    end

    def save
      reinitialize ApiClient.post(base_path, to_h)
    rescue => e
      begin
        cancel
      rescue RestClient::Exception
        #no-op
      end
      raise e
    end

    def created_at
      return nil unless super.present?
      Time.parse(super)
    end

    def cancelled_at
      return nil unless super.present?
      Time.parse(super)
    end

    def operations
      Collection.new(VdcOperation, super['data'], super['meta'])
    end

    def refunds
      Collection.new(Refund, super['data'], super['meta'])
    end
  end
end
